Package: tinyplot
Type: Package
Title: Lightweight Extension of the Base R Graphics System
Version: 0.6.1
Date: 2026-03-26
Authors@R: 
  c(
    person(
      given = "Grant",
      family = "McDermott",
      role = c("aut", "cre"),
      email = "contact@grantmcdermott.com",
      comment = c(ORCID = "0000-0001-7883-8573")
    ),
    person(
      given = "Vincent",
      family = "Arel-Bundock",
      role = "aut",
      email = "vincent.arel-bundock@umontreal.ca",
      comment = c(ORCID = "0000-0003-1995-6531")
    ),
    person(
      given = "Achim",
      family = "Zeileis",
      role = "aut",
      email = "Achim.Zeileis@R-project.org",
      comment = c(ORCID = "0000-0003-0918-3766")
     ),
    person(
      given = "Etienne",
      family = "Bacher",
      role = "ctb",
      email = "etienne.bacher@protonmail.com"
    )
  )
Description: Lightweight extension of the base R graphics system, with support
      for automatic legends, facets, themes, and various other enhancements.
License: Apache License (>= 2)
Depends: R (>= 4.0.0)
Imports: graphics, grDevices, stats, tools, utils
Suggests: fontquiver, png, rsvg, svglite (>= 2.2.0), tinytest,
        tinysnapshot (>= 0.0.3), knitr
Config/Needs/website: altdoc (>= 0.7.2), future.apply
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://grantmcdermott.com/tinyplot/
BugReports: https://github.com/grantmcdermott/tinyplot/issues
NeedsCompilation: no
Packaged: 2026-03-27 04:50:26 UTC; gmcd
Author: Grant McDermott [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7883-8573>),
  Vincent Arel-Bundock [aut] (ORCID:
    <https://orcid.org/0000-0003-1995-6531>),
  Achim Zeileis [aut] (ORCID: <https://orcid.org/0000-0003-0918-3766>),
  Etienne Bacher [ctb]
Maintainer: Grant McDermott <contact@grantmcdermott.com>
Repository: CRAN
Date/Publication: 2026-03-28 06:10:16 UTC
Built: R 4.6.0; ; 2026-04-26 15:00:45 UTC; unix
