# Generated by roxygen2: do not edit by hand

S3method(tinyplot,default)
S3method(tinyplot,density)
S3method(tinyplot,formula)
export(draw_legend)
export(get_saved_par)
export(plt)
export(plt_add)
export(tinylabel)
export(tinyplot)
export(tinyplot_add)
export(tinytheme)
export(tpar)
export(type_abline)
export(type_area)
export(type_barplot)
export(type_boxplot)
export(type_density)
export(type_errorbar)
export(type_function)
export(type_glm)
export(type_hist)
export(type_histogram)
export(type_hline)
export(type_jitter)
export(type_lines)
export(type_lm)
export(type_loess)
export(type_pointrange)
export(type_points)
export(type_polygon)
export(type_polypath)
export(type_qq)
export(type_rect)
export(type_ribbon)
export(type_ridge)
export(type_rug)
export(type_segments)
export(type_spineplot)
export(type_spline)
export(type_summary)
export(type_text)
export(type_violin)
export(type_vline)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,as.raster)
importFrom(grDevices,axisTicks)
importFrom(grDevices,cairo_pdf)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,convertColor)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.new)
importFrom(grDevices,dev.off)
importFrom(grDevices,extendrange)
importFrom(grDevices,gray.colors)
importFrom(grDevices,hcl)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,hcl.pals)
importFrom(grDevices,jpeg)
importFrom(grDevices,nclass.Sturges)
importFrom(grDevices,palette)
importFrom(grDevices,palette.colors)
importFrom(grDevices,palette.pals)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,recordGraphics)
importFrom(grDevices,svg)
importFrom(grDevices,xy.coords)
importFrom(graphics,Axis)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axTicks)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,boxplot)
importFrom(graphics,grconvertX)
importFrom(graphics,grconvertY)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.default)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,polypath)
importFrom(graphics,rasterImage)
importFrom(graphics,rect)
importFrom(graphics,rug)
importFrom(graphics,segments)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(stats,aggregate)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,bw.SJ)
importFrom(stats,bw.bcv)
importFrom(stats,bw.nrd)
importFrom(stats,bw.nrd0)
importFrom(stats,bw.ucv)
importFrom(stats,density)
importFrom(stats,dnorm)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,loess.control)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,na.omit)
importFrom(stats,ppoints)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(stats,spline)
importFrom(stats,terms)
importFrom(stats,weighted.mean)
importFrom(tools,file_ext)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,tail)
