#' Operations on package load
#' @importFrom utils globalVariables
#' @importFrom graphics legend
#' @param libname library name
#' @param pkgname package name name
#' @keywords internal
#' @noRd
.onLoad = function(libname, pkgname) {
  # https://stackoverflow.com/questions/12598242/global-variables-in-packages-in-r
  # https://stackoverflow.com/questions/49056642/r-how-to-make-variable-available-to-namespace-at-loading-time?noredirect=1&lq=1

  globalVariables(c(
    "add",
    "alpha",
    "asp",
    "axes",
    "bg",
    "bubble",
    "bubble_alpha",
    "bubble_bg_alpha",
    "bubble_cex",
    "by_continuous",
    "by_dep",
    "by_ordered",
    "cap",
    "cex",
    "cex_dep",
    "cex_fct_adj",
    "data",
    "datapoints",
    "dots",
    "draw",
    "facet",
    "facet.args",
    "facet_attr",
    "facet_bg",
    "facet_border",
    "facet_by",
    "facet_col",
    "facet_font",
    "facet_newlines",
    "facet_rect",
    "facet_text",
    "facets",
    "fill",
    "flip",
    "frame.plot",
    "has_legend",
    "height",
    "iby",
    "ifacet",
    "labels",
    "lighten",
    "lty",
    "lwd",
    "nfacet_cols",
    "nfacet_rows",
    "nfacets",
    "ngrps",
    "null_by",
    "null_facet",
    "null_palette",
    "null_xlim",
    "multi_legend",
    "legend",
    "lgnd_labs",
    "lgnd_cex",
    "has_cap",
    "has_sub",
    "legend_draw_flag",
    "multi_legend",
    "legend_args",
    "null_ylim",
    "lgby",
    "lgbub",
    "oxaxis",
    "oyaxis",
    "pch",
    "rev_x",
    "rev_y",
    "ribbon.alpha",
    "split_data",
    "tpars",
    "type",
    "type_info",
    "was_area_type",
    "weights",
    "width",
    "x",
    "x_by",
    "x_dep",
    "xaxb",
    "xaxl",
    "xaxs",
    "xaxt",
    "xlab",
    "xlabs",
    "xlim",
    "xlvls",
    "xmax",
    "xmax_dep",
    "xmin",
    "xmin_dep",
    "y",
    "y_dep",
    "yaxb",
    "yaxl",
    "yaxs",
    "yaxt",
    "ygroup",
    "ylab",
    "ylabs",
    "ylim",
    "ymax",
    "ymax_dep",
    "ymin",
    "ymin_dep"
  ))
}
