% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type_segments.R
\name{type_segments}
\alias{type_segments}
\title{Line segments plot type}
\usage{
type_segments()
}
\description{
Type function for plotting line segments.
}
\details{
Contrary to base \code{\link[graphics]{segments}}, line segments in
\link{tinyplot} must be specified using the \code{xmin}, \code{ymin},\code{xmax}, and \code{ymax}
arguments.
}
\examples{
# "segments" type convenience character string
tinyplot(
  xmin = c(0,.1), ymin = c(.2,1), xmax = c(1,.9), ymax = c(.75,0),
  type = "segments"
)

# Same result with type_segments()
tinyplot(
  xmin = c(0,.1), ymin = c(.2,1), xmax = c(1,.9), ymax = c(.75,0),
  type = type_segments()
)

}
