% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type_loess.R
\name{type_loess}
\alias{type_loess}
\title{Local polynomial regression plot type}
\usage{
type_loess(
  span = 0.75,
  degree = 2,
  family = "gaussian",
  control = loess.control(),
  se = TRUE,
  level = 0.95,
  weights = NULL
)
}
\arguments{
\item{span}{the parameter \eqn{\alpha} which controls the degree of
    smoothing.}

\item{degree}{the degree of the polynomials to be used, normally 1 or
    2. (Degree 0 is also allowed, but see the \sQuote{Note}.)}

\item{family}{a description of the error distribution and link
    function to be used in the model.  For \code{glm} this can be a
    character string naming a family function, a family function or the
    result of a call to a family function.  For \code{glm.fit} only the
    third option is supported.  (See \code{\link[stats]{family}} for details of
    family functions.)}

\item{control}{control parameters: see \code{\link[stats]{loess.control}}.}

\item{se}{logical. If \code{TRUE} (the default), confidence intervals are drawn.}

\item{level}{the confidence level required if \code{se = TRUE}. Default is 0.95.}

\item{weights}{an optional numeric vector of observation weights for the model
fit, of the same length as the number of data points. Weights can also be
supplied via the top-level \code{weights} argument of \code{\link{tinyplot}} (which is
evaluated with non-standard evaluation in the formula method, and takes
precedence if both are given).}
}
\description{
Type function for plotting a LOESS (LOcal regrESSion) fit.
Arguments are passed to \code{\link[stats]{loess}}.
}
\examples{
# "loess" type convenience string
tinyplot(dist ~ speed, data = cars, type = "loess")

# Use `type_loess()` to pass extra arguments for customization
tinyplot(dist ~ speed, data = cars, type = type_loess(span = 0.5, degree = 1))
}
