% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type_chull.R
\name{type_chull}
\alias{type_chull}
\title{Convex hull plot type}
\usage{
type_chull(density = NULL, angle = 45)
}
\arguments{
\item{density}{the density of shading lines, in lines per inch.  The
    default value of \code{NULL} means that no shading lines are drawn.
    A zero value of \code{density} means no shading nor filling whereas
    negative values and \code{NA} suppress shading (and so allow
    color filling).}

\item{angle}{the slope of shading lines, given as an angle
     in degrees (counter-clockwise).}
}
\description{
Type function for drawing convex hulls around grouped points.
Uses \code{\link[grDevices]{chull}} to compute the convex hull of each
group and draws the result as a polygon.
}
\examples{
# "chull" type convenience character string
tinyplot(Sepal.Length ~ Petal.Length | Species, data = iris, type = "chull")

# layer filled convex hull(s) on top of points
tinyplot(Sepal.Length ~ Petal.Length | Species, data = iris, theme = "basic")
tinyplot_add(type = "chull", fill = 0.2)

}
