% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/facet.R
\name{draw_facet_window}
\alias{draw_facet_window}
\alias{facet_layout}
\title{Draw facet windows}
\usage{
draw_facet_window(
  add,
  cex_fct_adj,
  facet.args,
  facet_newlines,
  facet_font,
  facet_rect,
  facet_text,
  facet_col,
  facet_bg,
  facet_border,
  facet,
  facets,
  ifacet,
  nfacets,
  nfacet_cols,
  nfacet_rows,
  axes,
  flip,
  frame.plot,
  oxaxis,
  oyaxis,
  xlabs,
  xlim,
  null_xlim,
  xaxt,
  xaxs,
  xaxb,
  xaxl,
  ylabs,
  ylim,
  null_ylim,
  yaxt,
  yaxs,
  yaxb,
  yaxl,
  rev_x = FALSE,
  rev_y = FALSE,
  asp,
  log,
  dots,
  draw,
  grid,
  has_legend,
  main,
  sub,
  cap,
  type,
  xlab,
  x,
  xmax,
  xmin,
  ylab,
  y,
  ymax,
  ymin,
  tpars = NULL,
  dynmar_computed = NULL,
  dl_overshoot = 0
)

facet_layout(settings)
}
\arguments{
\item{settings}{A list of settings as created by \code{tinyplot()}.}
}
\description{
Internal functions called from \code{tinyplot} in order to draw the
plot window with different facets, grids, axes, etc.

\code{facet_layout} determines the layout of the facets, based on a set of inputs.

\code{draw_facet_window} is the main workhorse function for setting the exterior
plot elements as part of a \code{tinyplot} call, including adjustment of margins
for dynamic themes, etc.
}
\keyword{internal}
