% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type_qq.R
\name{type_qq}
\alias{type_qq}
\title{Quantile-Quantile plot (QQ)}
\usage{
type_qq(distribution = qnorm)
}
\arguments{
\item{distribution}{Distribution function to use.}
}
\description{
Plots the theoretical quantiles of \code{x} on the horizontal axis
against observed values of \code{x} on the vertical axis.
}
\examples{
tinyplot(~mpg, data = mtcars, type = type_qq())

# suppress the line
tinyplot(~mpg, data = mtcars, lty = 0, type = type_qq())
}
