% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{restore_margin_outer}
\alias{restore_margin_outer}
\title{Restore outer margin defaults}
\usage{
restore_margin_outer()
}
\value{
NULL (called for side effect of resetting par("omd"))
}
\description{
Resets the outer margin display (omd) to default full device.
Used to clean up after legend drawing that may have adjusted margins.
}
\keyword{internal}
