% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type_glm.R
\name{type_glm}
\alias{type_glm}
\title{Generalized linear model plot type}
\usage{
type_glm(
  family = "gaussian",
  se = TRUE,
  level = 0.95,
  type = "response",
  weights = NULL
)
}
\arguments{
\item{family}{a description of the error distribution and link
    function to be used in the model.  For \code{glm} this can be a
    character string naming a family function, a family function or the
    result of a call to a family function.  For \code{glm.fit} only the
    third option is supported.  (See \code{\link[stats]{family}} for details of
    family functions.)}

\item{se}{logical. If TRUE, confidence intervals are drawn.}

\item{level}{the confidence level required.}

\item{type}{character, partial matching allowed.  Type of weights to
    extract from the fitted model object.  Can be abbreviated.}

\item{weights}{an optional numeric vector of observation weights for the model
fit, of the same length as the number of data points. Weights can also be
supplied via the top-level \code{weights} argument of \code{\link{tinyplot}} (which is
evaluated with non-standard evaluation in the formula method, and takes
precedence if both are given).}
}
\description{
Type function for plotting a generalized model fit.
Arguments are passed to \code{\link[stats]{glm}}.
}
\examples{
# "glm" type convenience string
tinyplot(am ~ mpg, data = mtcars, type = "glm")

# Use `type_glm()` to pass extra arguments for customization
tinyplot(am ~ mpg, data = mtcars, type = type_glm(family = "binomial"))
}
