% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend.R
\name{build_legend_args}
\alias{build_legend_args}
\title{Build legend arguments list}
\usage{
build_legend_args(
  legend_env,
  legend,
  legend_args,
  by_dep,
  lgnd_labs,
  labeller = NULL,
  type,
  pch,
  lty,
  lwd,
  col,
  bg,
  cex,
  gradient
)
}
\arguments{
\item{legend_env}{Legend environment to populate}

\item{legend}{Legend placement keyword or list}

\item{legend_args}{Additional legend arguments}

\item{by_dep}{The (deparsed) "by" grouping variable name}

\item{lgnd_labs}{The legend labels}

\item{labeller}{Character or function for formatting labels}

\item{type}{Plot type}

\item{pch}{Plotting character(s)}

\item{lty}{Line type(s)}

\item{lwd}{Line width(s)}

\item{col}{Color(s)}

\item{bg}{Background fill color(s)}

\item{cex}{Character expansion(s)}

\item{gradient}{Logical indicating gradient legend}
}
\value{
NULL (modifies legend_env in-place)
}
\description{
Constructs and configures the legend_args list by:
\itemize{
\item Sanitizing legend input
\item Setting defaults for all legend parameters
\item Computing positioning flags from original position (before transformation)
\item Adjusting position anchors for outer legends
\item Adjusting for special cases (gradient, horizontal, multi-column)
\item Populating legend_env with args and positioning flags
}
}
\keyword{internal}
