% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type_glm.R
\name{ci}
\alias{ci}
\title{Calculate confidence intervals}
\usage{
ci(estimate, std.error, conf.level, df, backtransform = identity)
}
\description{
Calculate confidence intervals
}
\keyword{internal}
