% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tinyplot-package.R
\docType{package}
\name{tinyplot-package}
\alias{tinyplot-package}
\title{tinyplot: Lightweight Extension of the Base R Graphics System}
\description{
Lightweight extension of the base R graphics system, with support for automatic legends, facets, themes, and various other enhancements.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://grantmcdermott.com/tinyplot/}
  \item Report bugs at \url{https://github.com/grantmcdermott/tinyplot/issues}
}

}
\author{
\strong{Maintainer}: Grant McDermott \email{contact@grantmcdermott.com} (\href{https://orcid.org/0000-0001-7883-8573}{ORCID})

Authors:
\itemize{
  \item Grant McDermott \email{contact@grantmcdermott.com} (\href{https://orcid.org/0000-0001-7883-8573}{ORCID})
  \item Vincent Arel-Bundock \email{vincent.arel-bundock@umontreal.ca} (\href{https://orcid.org/0000-0003-1995-6531}{ORCID})
  \item Achim Zeileis \email{Achim.Zeileis@R-project.org} (\href{https://orcid.org/0000-0003-0918-3766}{ORCID})
}

Other contributors:
\itemize{
  \item Etienne Bacher \email{etienne.bacher@protonmail.com} [contributor]
}

}
\keyword{internal}
