% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_multi.R
\name{prepare_legend_multi}
\alias{prepare_legend_multi}
\title{Prepare multi-legend specifications}
\usage{
prepare_legend_multi(settings)
}
\arguments{
\item{settings}{Settings environment from tinyplot}
}
\value{
NULL (modifies settings environment in-place)
}
\description{
Sets up multiple legend specifications for multi-legends
(e.g., color grouping + bubble size). Creates \code{lgby} and \code{lgbub} objects
that will be passed to draw_multi_legend().
}
\keyword{internal}
